// default.js 

// TODO:  edit the next 2 lines to point to the correct directories
var MATLAB32_LIB_PATH = "C:\\Program Files (x86)\\MATLAB\\R2013a\\extern\\lib\\win32\\microsoft\\;C:\\itpp-4.3.1\\lib\\";
var MATLAB32_INC_PATH = "C:\\Program Files (x86)\\MATLAB\\R2013a\\extern\\include\\;C:\\itpp-4.3.1\\";

var MATLAB64_LIB_PATH = "C:\\Program Files\\MATLAB\\R2014b\\extern\\lib\\win64\\microsoft\\;C:\\itpp-4.3.1\\lib\\";
var MATLAB64_INC_PATH = "C:\\Program Files\\MATLAB\\R2014b\\extern\\include\\;C:\\itpp-4.3.1\\";

function OnFinish(selProj, selObj)
{
	try
	{
		var strProjectPath = wizard.FindSymbol('PROJECT_PATH');
		var strProjectName = wizard.FindSymbol('PROJECT_NAME');

		selProj = CreateCustomProject(strProjectName, strProjectPath);
		selProj.ConfigurationManager.AddPlatform("x64", "Win32", true);
		AddCommonConfig(selProj, strProjectName);
		AddSpecificConfig(selProj, strProjectName);
		AddFilters(selProj);

		var InfFile = CreateCustomInfFile();
		AddFilesToCustomProj(selProj, strProjectName, strProjectPath, InfFile);
		PchSettings(selProj);
		InfFile.Delete();

		selProj.Object.Save();
	}
	catch(e)
	{
		if (e.description.length != 0)
			SetErrorInfo(e);
		return e.number
	}
}

function CreateCustomProject(strProjectName, strProjectPath)
{
	try
	{
		var strProjTemplatePath = wizard.FindSymbol('PROJECT_TEMPLATE_PATH');
		var strProjTemplate = '';
		strProjTemplate = strProjTemplatePath + '\\default.vcxproj';

		var Solution = dte.Solution;
		var strSolutionName = "";
		if (wizard.FindSymbol("CLOSE_SOLUTION"))
		{
			Solution.Close();
			strSolutionName = wizard.FindSymbol("VS_SOLUTION_NAME");
			if (strSolutionName.length)
			{
				var strSolutionPath = strProjectPath.substr(0, strProjectPath.length - strProjectName.length);
				Solution.Create(strSolutionPath, strSolutionName);
			}
		}

		var strProjectNameWithExt = '';
		strProjectNameWithExt = strProjectName + '.vcxproj';

		var oTarget = wizard.FindSymbol("TARGET");
		var prj;
		if (wizard.FindSymbol("WIZARD_TYPE") == vsWizardAddSubProject)  // vsWizardAddSubProject
		{
			var prjItem = oTarget.AddFromTemplate(strProjTemplate, strProjectNameWithExt);
			prj = prjItem.SubProject;
		}
		else
		{
			prj = oTarget.AddFromTemplate(strProjTemplate, strProjectPath, strProjectNameWithExt);
		}
		return prj;
	}
	catch(e)
	{
		throw e;
	}
}

function AddFilters(proj)
{
	try
	{
		// Add the folders to your project
		var strSrcFilter = wizard.FindSymbol('SOURCE_FILTER');
		var group = proj.Object.AddFilter('Source Files');
		group.Filter = strSrcFilter;
	}
	catch(e)
	{
		throw e;
	}
}

function AddSpecificConfig(proj, strProjectName){
        try{
			  ConfigurePlatformFlavor(proj, strProjectName, "x64",   "Debug");
		      ConfigurePlatformFlavor(proj, strProjectName, "x64",   "Release");
			  ConfigurePlatformFlavor(proj, strProjectName, "Win32", "Debug");
		      ConfigurePlatformFlavor(proj, strProjectName, "Win32", "Release");
        }
		catch(e)
		{
                throw e;
        }
}

function ConfigurePlatformFlavor(proj, strProjectName, strPlatform, strFlavor)
{
	try
	{
	    // Assumes that the platform already exists
	    var config = proj.Object.Configurations(strFlavor + "|" + strPlatform);
		
		if (IsX86(strPlatform)){
			if (strFlavor == "Debug") {
			////////////////////////////////////
			// debug configuration
			config.IntermediateDirectory = "$(Platform)\\$(Configuration)\\";
			config.OutputDirectory = "$(SolutionDir)$(Platform)\\$(Configuration)\\";
			config.ConfigurationType = ConfigurationTypes.typeDynamicLibrary;
			config.FileTools(1).TargetExt = ".mexw32";
			
			// compiler settings
			var CLTool = config.Tools('VCCLCompilerTool');
			
			CLTool.UsePrecompiledHeader = false;
			CLTool.AdditionalIncludeDirectories = MATLAB32_INC_PATH;
			
			var strDefines = "WIN32;_DEBUG;_WINDOWS;_USRDLL;_MBCS;_CRT_SECURE_NO_DEPRECATE;";
			var exportName = strProjectName + '_EXPORTS';
			strDefines += exportName;
			CLTool.PreprocessorDefinitions = strDefines;
			
			CLTool.RuntimeLibrary = rtMultiThreadedDebug;
			
			// linker settings
			var LinkTool = config.Tools('VCLinkerTool');
			
			LinkTool.AdditionalDependencies = "itpp_debug_win32.lib lapack_win32_MTd.lib libfftw3-3_win32.lib blas_win32_MTd.lib libmex.lib libmx.lib";
			LinkTool.AdditionalLibraryDirectories = MATLAB32_LIB_PATH;
			LinkTool.OutputFile = "$(OutDir)$(TargetName)" + ".mexw32";
			
			var moduleFile = strProjectName + '.def';
			LinkTool.ModuleDefinitionFile = moduleFile;
		} else {
			/////////////////////////////////////////
			// release configurations
			config.IntermediateDirectory = "$(Platform)\\$(Configuration)\\";
			config.OutputDirectory = "$(SolutionDir)$(Platform)\\$(Configuration)\\";
			config.ConfigurationType = ConfigurationTypes.typeDynamicLibrary;
			config.FileTools(1).TargetExt = ".mexw32";
			
			// compiler settings
			var CLTool = config.Tools('VCCLCompilerTool');
			CLTool.UsePrecompiledHeader = false;
			CLTool.AdditionalIncludeDirectories = MATLAB32_INC_PATH;
			
			var strDefines = "WIN32;NDEBUG;_WINDOWS;_USRDLL;_MBCS;_CRT_SECURE_NO_DEPRECATE;";
			var exportName = strProjectName + '_EXPORTS';
			strDefines += exportName;
			CLTool.PreprocessorDefinitions = strDefines;
			
			CLTool.RuntimeLibrary = rtMultiThreaded;
			
			// linker settings
			var LinkTool = config.Tools('VCLinkerTool');
			LinkTool.AdditionalDependencies = "itpp_rel_win32.lib lapack_win32_MT.lib libfftw3-3_win32.lib blas_win32_MT.lib libmex.lib libmx.lib";
			LinkTool.AdditionalLibraryDirectories = MATLAB32_LIB_PATH;
			LinkTool.OutputFile = "$(OutDir)$(TargetName)" + ".mexw32";

			var moduleFile = strProjectName + '.def';
			LinkTool.ModuleDefinitionFile = moduleFile;
		}
	}
    if (IsX64(strPlatform)){
		if (strFlavor == "Debug") {
			////////////////////////////////////
			// debug configuration
			config.IntermediateDirectory = "$(Platform)\\$(Configuration)\\";
			config.OutputDirectory = "$(SolutionDir)$(Platform)\\$(Configuration)\\";
			config.ConfigurationType = ConfigurationTypes.typeDynamicLibrary;
			config.FileTools(1).TargetExt = ".mexw64";
						
    
			// compiler settings
			var CLTool = config.Tools('VCCLCompilerTool');
			
			CLTool.UsePrecompiledHeader = false;
			CLTool.AdditionalIncludeDirectories = MATLAB64_INC_PATH;
			
			var strDefines = "WIN32;_DEBUG;_WINDOWS;_USRDLL;_MBCS;_CRT_SECURE_NO_DEPRECATE;";
			var exportName = strProjectName + '_EXPORTS';
			strDefines += exportName;
			CLTool.PreprocessorDefinitions = strDefines;
			
			CLTool.RuntimeLibrary = rtMultiThreadedDebug;
			
			// linker settings
			var LinkTool = config.Tools('VCLinkerTool');
			
			LinkTool.AdditionalDependencies = "itpp_debug_win64.lib lapack_win64_MTd.lib libfftw3-3_win64.lib blas_win64_MTd.lib libmex.lib libmx.lib";
			LinkTool.AdditionalLibraryDirectories = MATLAB64_LIB_PATH;
			LinkTool.OutputFile = "$(OutDir)$(TargetName)" + ".mexw64";
			
			var moduleFile = strProjectName + '.def';
			LinkTool.ModuleDefinitionFile = moduleFile;
		} else {
			/////////////////////////////////////////
			// release configurations
			config.IntermediateDirectory = "$(Platform)\\$(Configuration)\\";
			config.OutputDirectory = "$(SolutionDir)$(Platform)\\$(Configuration)\\";
			config.ConfigurationType = ConfigurationTypes.typeDynamicLibrary;
			config.FileTools(1).TargetExt = ".mexw64";
		
			// compiler settings
			var CLTool = config.Tools('VCCLCompilerTool');
			CLTool.UsePrecompiledHeader = false;
			CLTool.AdditionalIncludeDirectories = MATLAB64_INC_PATH;
			
			var strDefines = "WIN32;NDEBUG;_WINDOWS;_USRDLL;_MBCS;_CRT_SECURE_NO_DEPRECATE;";
			var exportName = strProjectName + '_EXPORTS';
			strDefines += exportName;
			CLTool.PreprocessorDefinitions = strDefines;
			CLTool.EnableIntrinsicFunctions = true;
			CLTool.OmitFramePointers = false;
			CLTool.WholeProgramOptimization = true;
			
			CLTool.RuntimeLibrary = rtMultiThreaded;
			
			// linker settings
			var LinkTool = config.Tools('VCLinkerTool');
			LinkTool.AdditionalDependencies = "itpp_rel_win64.lib lapack_win64_MT.lib libfftw3-3_win64.lib blas_win64_MT.lib libmex.lib libmx.lib";
			LinkTool.AdditionalLibraryDirectories = MATLAB64_LIB_PATH;
			LinkTool.OutputFile = "$(OutDir)$(TargetName)" + ".mexw64";

			var moduleFile = strProjectName + '.def';
			LinkTool.ModuleDefinitionFile = moduleFile;
		}
		}		
	}
	catch(e)
	{
		throw e;
	}
}

function PchSettings(proj)
{
	// TODO: specify pch settings
}

function DelFile(fso, strWizTempFile)
{
	try
	{
		if (fso.FileExists(strWizTempFile))
		{
			var tmpFile = fso.GetFile(strWizTempFile);
			tmpFile.Delete();
		}
	}
	catch(e)
	{
		throw e;
	}
}

function CreateCustomInfFile()
{
	try
	{
		var fso, TemplatesFolder, TemplateFiles, strTemplate;
		fso = new ActiveXObject('Scripting.FileSystemObject');

		var TemporaryFolder = 2;
		var tfolder = fso.GetSpecialFolder(TemporaryFolder);
		var strTempFolder = tfolder.Drive + '\\' + tfolder.Name;

		var strWizTempFile = strTempFolder + "\\" + fso.GetTempName();

		var strTemplatePath = wizard.FindSymbol('TEMPLATES_PATH');
		var strInfFile = strTemplatePath + '\\Templates.inf';
		wizard.RenderTemplate(strInfFile, strWizTempFile);

		var WizTempFile = fso.GetFile(strWizTempFile);
		return WizTempFile;
	}
	catch(e)
	{
		throw e;
	}
}

function GetTargetName(strName, strProjectName)
{
	try
	{
		// TODO: set the name of the rendered file based on the template filename
		var strTarget = strName;

		if (strName == 'readme.txt')
			strTarget = 'ReadMe.txt';
		
		if (strName == 'mexFunction.cpp')
			strTarget = strProjectName + '.cpp';
						
		if (strName == 'module.def')
			strTarget = strProjectName + '.def';

		return strTarget; 
	}
	catch(e)
	{
		throw e;
	}
}

function AddFilesToCustomProj(proj, strProjectName, strProjectPath, InfFile)
{
	try
	{
		var projItems = proj.ProjectItems

		var strTemplatePath = wizard.FindSymbol('TEMPLATES_PATH');

		var strTpl = '';
		var strName = '';

		var strTextStream = InfFile.OpenAsTextStream(1, -2);
		while (!strTextStream.AtEndOfStream)
		{
			strTpl = strTextStream.ReadLine();
			if (strTpl != '')
			{
				strName = strTpl;
				var strTarget = GetTargetName(strName, strProjectName);
				var strTemplate = strTemplatePath + '\\' + strTpl;
				var strFile = strProjectPath + '\\' + strTarget;

				var bCopyOnly = false;  //"true" will only copy the file from strTemplate to strTarget without rendering/adding to the project
				var strExt = strName.substr(strName.lastIndexOf("."));
				if(strExt==".bmp" || strExt==".ico" || strExt==".gif" || strExt==".rtf" || strExt==".css")
					bCopyOnly = true;
				wizard.RenderTemplate(strTemplate, strFile, bCopyOnly);
				proj.Object.AddFile(strFile);
			}
		}
		strTextStream.Close();
	}
	catch(e)
	{
		throw e;
	}
}
function IsX86(strPlatform)
{
	return (strPlatform == "Win32");
}

function IsX64(strPlatform)
{
	return (strPlatform == "x64");
}

function IsDebugFlavor(strFlavor)
{
	return (strFlavor == "Debug");
}

function IsReleaseFlavor(strFlavor)
{
	return (strFlavor == "Release");
}